/*
  This software is available to you under a choice of one of two
  licenses.  You may choose to be licensed under the terms of the GNU
  General Public License (GPL) Version 2, available at
  <http://www.fsf.org/copyleft/gpl.html>, or the OpenIB.org BSD
  license, available in the LICENSE.TXT file accompanying this
  software.  These details are also available at
  <http://openib.org/license.html>.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.

  Copyright (c) 2004 Topspin Communications.  All rights reserved.
  Copyright (c) 2005 Mellanox Technologies Ltd.  All rights reserved.

  $Id: libsdp.h 1897 2005-02-24 15:43:29Z mst $
*/

#include <netinet/in.h>

/*
 * SDP specific includes
 */
#include "sdp_inet.h"

/* --------------------------------------------------------------------- */
/* library static and global variables                                   */
/* --------------------------------------------------------------------- */
typedef enum {
  MATCH_PROGRAM_NAME = 1,
  MATCH_LISTEN_IP_PORT,
  MATCH_DESTINATION_IP_PORT,
} match_type_t;

struct match_expression {
  struct match_expression *prev, *next;

  match_type_t type;

  union {
    struct {
      struct in_addr ip;
      unsigned char prefixlen;		/* length of CIDR prefix (ie /24) */
      unsigned short sport, eport;	/* start port - end port, inclusive */
    } ip_port;
    char *program_name;
  } data;
};

struct match_group {
  struct match_group *prev, *next;

  struct match_expression *head, *tail;
};

extern struct match_group *__sdp_match_groups_head;
extern struct match_group *__sdp_match_groups_tail;
extern struct match_group *__sdp_match_both_groups_head;
extern struct match_group *__sdp_match_both_groups_tail;

#define SDP_NETMASK(n) ((n == 0) ? 0 : ~((1UL<<(32 - n)) - 1))

/* match.c */
int __sdp_match_connect(
                        const struct sockaddr_in *sin
                        );
int __sdp_match_listen(
                       const struct sockaddr_in *sin, int both
                       );

/* config.c */
int __sdp_config_empty(
                       void
                       );

int __sdp_read_config(
                      char *config_file
                      );

/* log.c */
void __sdp_log(
               int level,
               char *format,
               ...
               );

void __sdp_log_set_min_level(
                             int level
                             );

int __sdp_log_set_log_stderr(
                             void
                             );

int __sdp_log_set_log_syslog(
                             void
                             );

int __sdp_log_set_log_file(
                           char *filename
                           );
