/*
  This software is available to you under a choice of one of two
  licenses.  You may choose to be licensed under the terms of the GNU
  General Public License (GPL) Version 2, available at
  <http://www.fsf.org/copyleft/gpl.html>, or the OpenIB.org BSD
  license, available in the LICENSE.TXT file accompanying this
  software.  These details are also available at
  <http://openib.org/license.html>.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.

  Copyright (c) 2004 Topspin Communications.  All rights reserved.
  Copyright (c) 2005 Mellanox Technologies Ltd.  All rights reserved.

  $Id: match.c 1897 2005-02-24 15:43:29Z mst $
*/

/*
 * system includes
 */
#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fnmatch.h>
/*
 * SDP specific includes
 */
#include "libsdp.h"

/* --------------------------------------------------------------------- */
/* library static and global variables                                   */
/* --------------------------------------------------------------------- */
extern char *program_invocation_name, *program_invocation_short_name;

static int match_ip_port(
                         struct match_expression *expression,
	                 const struct sockaddr_in *sin
                         )
{
  unsigned short port = ntohs(sin->sin_port);

  __sdp_log(3, "MATCH:     matching %d.%d.%d.%d:%d to %d.%d.%d.%d/%d:%d-%d\n",
    sin->sin_addr.s_addr & 0xff,
    (sin->sin_addr.s_addr >> 8) & 0xff,
    (sin->sin_addr.s_addr >> 16) & 0xff,
    (sin->sin_addr.s_addr >> 24) & 0xff,
    port,
    expression->data.ip_port.ip.s_addr & 0xff,
    (expression->data.ip_port.ip.s_addr >> 8) & 0xff,
    (expression->data.ip_port.ip.s_addr >> 16) & 0xff,
    (expression->data.ip_port.ip.s_addr >> 24) & 0xff,
    expression->data.ip_port.prefixlen,
    expression->data.ip_port.sport,
    expression->data.ip_port.eport);

  if (port < expression->data.ip_port.sport ||
      port > expression->data.ip_port.eport)
    return 0;

  if (expression->data.ip_port.ip.s_addr !=
      (sin->sin_addr.s_addr & htonl(SDP_NETMASK(expression->data.ip_port.prefixlen))))
    return 0;

  return 1;
}

static int match_program_name(
                              struct match_expression *expression
                              )
{
  return !fnmatch(expression->data.program_name, program_invocation_short_name, 0);
}

int __sdp_match_listen(
                       const struct sockaddr_in *sin,int both
                       )
{
  struct match_group *group;
  struct match_group *head;
  if (both) {
    head=__sdp_match_both_groups_head;
  } else {
    head= __sdp_match_groups_head;
  };
  for (group = head; group; group = group->next) {
    struct match_expression *expression;
    int match = group->head != NULL;

    for (expression = group->head; expression && match; expression = expression->next) {
      switch (expression->type) {
      case MATCH_LISTEN_IP_PORT:
        match = match_ip_port(expression, sin);
        break;
      case MATCH_PROGRAM_NAME:
        match = match_program_name(expression);
        break;
      case MATCH_DESTINATION_IP_PORT:
      default:
        match = 0;
        break;
      }
    }

    if (match)
      return 1;
  }

  return 0;
}






int __sdp_match_connect(
                        const struct sockaddr_in *sin
                        )
{
  struct match_group *group;

  for (group = __sdp_match_groups_head; group; group = group->next) {
    struct match_expression *expression;
    int match = group->head != NULL;

    for (expression = group->head; expression && match; expression = expression->next) {
      switch (expression->type) {
      case MATCH_DESTINATION_IP_PORT:
        match = match_ip_port(expression, sin);
        break;
      case MATCH_PROGRAM_NAME:
        match = match_program_name(expression);
        break;
      case MATCH_LISTEN_IP_PORT:
      default:
        match = 0;
        break;
      }
    }

    if (match)
      return 1;
  }

  return 0;
}
